"""Test dataloaders."""
import argparse

from mpi4py import MPI

from dataloaders import load_data_csv_parallel, load_data_csv_root
from distributed_forest import MPITimer

if __name__ == "__main__":
    # Set up communicator.
    comm = MPI.COMM_WORLD
    rank, size = comm.rank, comm.size

    # Data file specifications
    header_lines = 0
    sep = ","
    encoding = "utf-8"
    path_to_data = "/pfs/work7/workspace/scratch/ku4408-VL-ScalableAI/data/SUSY.csv"

    parser = argparse.ArgumentParser(prog="Distributed Random Forest")
    parser.add_argument(
        "--train_split",
        type=float,
        default=0.75,
        help="The fraction of the dataset used for training.",
    )
    parser.add_argument(
        "--verbose",
        action="store_true",
        help="The verbosity.",
    )
    args = parser.parse_args()

    seed = size

    print(f"[{rank}/{size}]: Loading data using truly parallel dataloader...")
    with MPITimer(comm, name="truly parallel data loading"):
        (
            train_samples_local_par,
            train_targets_local_par,
            test_samples_par,
            test_targets_par,
        ) = load_data_csv_parallel(
            path_to_data, header_lines, comm, args.train_split, args.verbose
        )

    print(f"[{rank}/{size}]: Loading data using root-based dataloader...")
    with MPITimer(comm, name="root-based data loading"):
        (
            train_samples_local_root,
            train_targets_local_root,
            test_samples_root,
            test_targets_root,
        ) = load_data_csv_root(
            path_to_data, header_lines, comm, seed, args.train_split, args.verbose
        )

    print(
        f"[{rank}/{size}]: DONE.\n"
        f"Parallel: Local train samples / targets have shapes "
        f"{train_samples_local_par.shape} / {train_targets_local_par.shape}.\n"
        f"Parallel: Global test samples / targets have shapes {test_samples_par.shape} / {test_targets_par.shape}.\n"
        f"Root: Local train samples / targets have shapes "
        f"{train_samples_local_root.shape} / {train_targets_local_root.shape}.\n"
        f"Root: Global test samples / targets have shapes {test_samples_root.shape} / {test_targets_root.shape}."
    )
